searchData={"items":[{"type":"module","title":"Orb","doc":"Write WebAssembly modules with Elixir.\n\nWebAssembly is a low-level language. You work with integers and floats, can perform operations on them like adding or multiplication, and then read and write those values to a block of memory. There’s no concept of a “string” or an “array”, let alone a “hash map” or “HTTP request”.\n\nThat’s where a library like Orb can help out. It takes full advantage of Elixir’s language features by becoming a compiler for WebAssembly. You can define WebAssembly modules in Elixir for “string” or “hash map”, and compose them together into a final module.\n\nThat WebAssembly module can then run in every major application environment: browsers, servers, the edge, and mobile devices like phones, tablets & laptops. This story is still being developed, but I believe like other web standards like JSON, HTML, and HTTP, that WebAssembly will become a first-class citizen on any platform. It’s Turing-complete, designed to be backwards compatible, fast, and works almost everywhere.","ref":"Orb.html"},{"type":"module","title":"Example - Orb","doc":"Let’s create a module that calculates the average of a set of numbers.\n\nWebAssembly modules can have state. Here will have two pieces of state: a total `count` and a running `tally`. These are stored as **globals**. (If you are familiar with object-oriented programming, you can think of them as instance variables).\n\nOur module will export two functions: `insert` and `calculate_mean`. These two functions will work with the `count` and `tally` globals.\n\n```elixir\ndefmodule CalculateMean do\n  use Orb\n\n  global do\n    @count 0\n    @tally 0\n  end\n\n  defw insert(element: I32) do\n    @count = @count + 1\n    @tally = @tally + element\n  end\n\n  defw calculate_mean(), I32 do\n    @tally / @count\n  end\nend\n```\n\nOne thing you’ll notice is that we must specify the type of function parameters and return values. Our `insert` function accepts a 32-bit integer, denoted using `I32`. It returns no value, while `calculate_mean` is annotated to return a 32-bit integer.\n\nWe get to write math with the intuitive `+` and `/` operators. Let’s see the same module without the magic: no math operators and without `@` conveniences for working with globals:\n\n```elixir\ndefmodule CalculateMean do\n  use Orb\n\n  I32.global(count: 0, tally: 0)\n\n  defw insert(element: I32) do\n    Orb.Instruction.Global.Set.new(Orb.I32, :count, I32.add(Orb.Instruction.Global.Get.new(Orb.I32, :count), 1))\n    Orb.Instruction.Global.Set.new(Orb.I32, :tally, I32.add(Orb.Instruction.Global.Get.new(Orb.I32, :tally), element))\n  end\n\n  defw calculate_mean(), I32 do\n    I32.div_s(Orb.Instruction.Global.Get.new(Orb.I32, :tally), Orb.Instruction.Global.Get.new(Orb.I32, :count))\n  end\nend\n```\n\nThis is the exact same logic as before. In fact, this is what the first version expands to. Orb adds “sugar syntax” to make authoring WebAssembly nicer, to make it feel like writing Elixir or Ruby.","ref":"Orb.html#module-example"},{"type":"module","title":"Functions - Orb","doc":"In Elixir you define functions publicly available outside the module with `def/1`, and functions private to the module with `defp/1`. Orb follows the same suffix convention with `defw/2` and `defwp/2`.\n\nConsumers of your WebAssembly module will only be able to call exported functions defined using `defw/2`. Making a function public in WebAssembly is known as “exporting”.","ref":"Orb.html#module-functions"},{"type":"module","title":"Stack based - Orb","doc":"While it looks like Elixir, there are some key differences between it and programs written in Orb. The first is that state is mutable. While immutability is one of the best features of Elixir, in WebAssembly variables are mutable because raw computer memory is mutable.\n\nThe second key difference is that WebAssembly is stack based. Every function has an implicit stack of values that you can push and pop from. This paradigm allows WebAssembly runtimes to efficiently optimize for raw CPU registers whilst not being platform specific.\n\nIn Elixir when you write:\n\n```elixir\ndef example() do\n  1\n  2\n  3\nend\n```\n\nThe first two lines with `1` and `2` are inert — they have no effect — and the result from the function is the last line `3`.\n\nIn WebAssembly / Orb when you write the same sort of thing:\n\n```elixir\ndefw example() do\n  1\n  2\n  3\nend\n```\n\nThen what’s happening is that we are pushing `1` onto the stack, then `2`, and then `3`. Now the stack has three items on it. Which will become our return value: a tuple of 3 integers. (Our function has no return type specified, so this will be an error if you attempted to compile the resulting module).\n\nSo the correct return type from this function would be a tuple of three integers:\n\n```elixir\ndefw example(), {I32, I32, I32} do\n  1\n  2\n  3\nend\n```\n\nIf you prefer, Orb allows you to be explicit with your stack pushes with `Orb.Stack.push/1`:\n\n```elixir\ndefw example(), {I32, I32, I32} do\n  Orb.Stack.push(1)\n  Orb.Stack.push(2)\n  Orb.Stack.push(3)\nend\n```\n\nYou can use the stack to unlock novel patterns, but for the most part Orb avoids the need to interact with it. It’s just something to keep in mind if you are used to lines of code with simple values not having any side effects.","ref":"Orb.html#module-stack-based"},{"type":"module","title":"Locals - Orb","doc":"Locals are variables that live for the lifetime of a function. They must be specified upfront with their type alongside the function’s definition, and are initialized to zero.\n\nHere we have two locals: `under?` and `over?`, both 32-bit integers. We can set their value and then read them again at the bottom of the function.\n\n```elixir\ndefmodule WithinRange do\n  use Orb\n\n  defw validate(num: I32), I32, under?: I32, over?: I32 do\n    under? = num < 1\n    over? = num > 255\n\n    not (under? or over?)\n  end\nend\n```","ref":"Orb.html#module-locals"},{"type":"module","title":"Globals - Orb","doc":"Globals are like locals, but live for the duration of the entire running module’s life. Their initial type and value are specified upfront.\n\nGlobals by default are internal: nothing outside the module can see them. They can be exported to expose them to the outside world.\n\n```elixir\ndefmodule GlobalExample do\n  use Orb\n\n  global do # :mutable by default\n    @some_internal_global 99\n  end\n\n  global :readonly do\n    @some_internal_constant 99\n  end\n\n  global :export_readonly do\n    @some_public_constant 1001\n  end\n\n  global :export_mutable do\n    @some_public_variable 42\n  end\n\n  # You can define multiple globals at once:\n  global do\n    @magic_number_a 99\n    @magic_number_b 12\n    @magic_number_c -5\n  end\nend\n```\n\nYou can read or write to a global within `defw` using the `@` prefix:\n\n```elixir\ndefmodule Counter do\n  use Orb\n\n  global do\n    @counter 0\n  end\n\n  defw increment() do\n    @counter = @counter + 1\n  end\nend\n```\n\nWhen you use `Orb.global/1` an Elixir module attribute with the same name and initial value is also defined for you:application\n\n```elixir\ndefmodule DeepThought do\n  use Orb\n\n  global do\n    @meaning_of_life 42\n  end\n\n  def get_meaning_of_life_elixir() do\n    @meaning_of_life\n  end\n\n  defw get_meaning_of_life_wasm(), I32 do\n    @meaning_of_life\n  end\nend\n```","ref":"Orb.html#module-globals"},{"type":"module","title":"Memory - Orb","doc":"WebAssembly provides a buffer of memory when you need more than a handful global integers or floats. This is a contiguous array of random-access memory which you can freely read and write to.\n\n#","ref":"Orb.html#module-memory"},{"type":"module","title":"Pages - Orb","doc":"WebAssembly Memory comes in 64 KiB segments called pages. You use some multiple of these 64 KiB (64 * 1024 = 65,536 bytes) pages.\n\nBy default your module will have **no** memory, so you must specify how much memory you want upfront.\n\nHere’s an example with 16 pages (1 MiB) of memory:\n\n```elixir\ndefmodule Example do\n  use Orb\n\n  Memory.pages(16)\nend\n```\n\n#","ref":"Orb.html#module-pages"},{"type":"module","title":"Reading & writing memory - Orb","doc":"To read from memory, you can use the `Memory.load/2` function. This loads a value at the given memory address. Addresses are themselves 32-bit integers. This mean you can perform pointer arithmetic to calculate whatever address you need to access.\n\nHowever, this can prove unsafe as it’s easy to calculate the wrong address and corrupt your memory. For this reason, Orb provides higher level constructs for making working with memory pointers more pleasant, which are detailed later on.\n\n```elixir\ndefmodule Example do\n  use Orb\n\n  Memory.pages(1)\n\n  defw get_int32(), I32 do\n    Memory.load!(I32, 0x100)\n  end\n\n  defw set_int32(value: I32) do\n    Memory.store!(I32, 0x100, value)\n  end\nend\n```\n\n#","ref":"Orb.html#module-reading-writing-memory"},{"type":"module","title":"Initializing memory with data - Orb","doc":"You can populate the initial memory of your module using `Orb.Memory.initial_data!/2`. This accepts an memory offset and the string to write there.\n\n```elixir\ndefmodule MimeTypeDataExample do\n  use Orb\n\n  Memory.pages(1)\n\n  Memory.initial_data!(0x100, \"text/html\")\n  Memory.initial_data!(0x200, \"\"\"\n    <!doctype html>\n     \n     Hello world \n    \"\"\")\n\n  defw get_mime_type(), I32 do\n    0x100\n  end\n\n  defw get_body(), I32 do\n    0x200\n  end\nend\n```\n\nHaving to manually allocate and remember each memory offset is a pain, so Orb provides conveniences which are detailed in the next section.","ref":"Orb.html#module-initializing-memory-with-data"},{"type":"module","title":"Strings constants - Orb","doc":"You can use constant strings directly. These will be extracted as initial data definitions at the start of the WebAssembly module, and their memory offsets substituted in their place.\n\nEach string is packed together for maximum efficiency of memory space. Strings are deduplicated, so you can use the same string constant multiple times and a single allocation will be made.\n\nString constants in Orb are nul-terminated.\n\n```elixir\ndefmodule MimeTypeStringExample do\n  use Orb\n\n  Memory.pages(1)\n\n  defw get_mime_type(), Str do\n    \"text/html\"\n  end\n\n  defw get_body(), Str do\n    \"\"\"\n    <!doctype html>\n     \n     Hello world \n    \"\"\"\n  end\nend\n```","ref":"Orb.html#module-strings-constants"},{"type":"module","title":"Control flow - Orb","doc":"Orb supports control flow with `if`, `block`, and `loop` statements.\n\n#","ref":"Orb.html#module-control-flow"},{"type":"module","title":"If statements - Orb","doc":"If you want to run logic conditionally, use an `if` statement.\n\n```elixir\nif @party_mode? do\n  music_volume = 100\nend\n```\n\nYou can add an `else` clause:\n\n```elixir\nif @party_mode? do\n  music_volume = 100\nelse\n  music_volume = 30\nend\n```\n\nIf you want a ternary operator (e.g. to map from one value to another), you can use `if` when both clauses push the same type:\n\n```elixir\nmusic_volume = if @party_mode? do\n  i32(100)\nelse\n  i32(30)\nend\n```\n\nThese can be written on single line too:\n\n```elixir\nmusic_volume = I32.if(@party_mode?, do: i32(100), else: i32(30))\n```\n\n#","ref":"Orb.html#module-if-statements"},{"type":"module","title":"Loops - Orb","doc":"Loops look like the familiar construct in other languages like JavaScript, with two key differences: each loop has a name, and loops by default stop unless you explicitly tell them to continue.\n\n```elixir\ni = 0\nloop CountUp do\n  i = i + 1\n\n  CountUp.continue(if: i < 10)\nend\n```\n\nEach loop is named, so if you nest them you can specify which particular one to continue.\n\n```elixir\ntotal_weeks = 10\nweekday_count = 7\nweek = 0\nweekday = 0\nloop Weeks do\n  loop Weekdays do\n    # Do something here with week and weekday\n\n    weekday = weekday + 1\n    Weekdays.continue(if: weekday < weekday_count)\n  end\n\n  week = week + 1\n  Weeks.continue(if: week < total_weeks)\nend\n```\n\n##","ref":"Orb.html#module-loops"},{"type":"module","title":"Iterators - Orb","doc":"Iterators are an upcoming feature, currently part of SilverOrb that will hopefully become part of Orb itself.\n\n#","ref":"Orb.html#module-iterators"},{"type":"module","title":"Blocks - Orb","doc":"Blocks provide a structured way to skip code.\n\n```elixir\nControl.block Validate do\n  Validate.break(if: i < 0)\n\n  # Do something with i\nend\n```\n\nBlocks can have a type.\n\n```elixir\nControl.block Double, I32 do\n  if i < 0 do\n    push(0)\n    Double.break()\n  end\n\n  push(i * 2)\nend\n```","ref":"Orb.html#module-blocks"},{"type":"module","title":"Calling other functions - Orb","doc":"When you use `defw`, a corresponding Elixir function is defined for you using `def`.\n\n```elixir\ndefw magic_number(), I32 do\n  42\nend\n```\n\n```elixir\ndefw some_example(), n: I32 do\n  n = magic_number()\nend\n```","ref":"Orb.html#module-calling-other-functions"},{"type":"module","title":"Composing modules with `Orb.include/1` - Orb","doc":"The WebAssembly functions from one module reused in another using `Orb.include/1`.\n\nHere’s an example of module A’s square function being included by module B:\n\n```elixir\ndefmodule A do\n  use Orb\n\n  defw square(n: I32), I32 do\n    n * n\n  end\nend\n\ndefmodule B do\n  use Orb\n\n  # Copies all WebAssembly functions defined in A into this module.\n  Orb.include(A)\n\n  defw example(n: I32), I32 do\n    # Now we can call functions on A.\n    A.square(42)\n  end\nend\n```","ref":"Orb.html#module-composing-modules-with-orb-include-1"},{"type":"macro","title":"Orb.export/1","doc":"Makes the globals inside exported.","ref":"Orb.html#export/1"},{"type":"macro","title":"Orb.functype/2","doc":"","ref":"Orb.html#functype/2"},{"type":"macro","title":"Orb.global/2","doc":"Declare WebAssembly globals.\n\n`mode` can be :readonly, :mutable, :export_readonly, or :export_mutable. The default is :mutable.","ref":"Orb.html#global/2"},{"type":"macro","title":"Examples - Orb.global/2","doc":"```elixir\ndefmodule GlobalExample do\n  use Orb\n\n  global do # :mutable by default\n    @some_internal_global 99\n  end\n\n  global :readonly do\n    @some_internal_constant 99\n  end\n\n  global :export_readonly do\n    @some_public_constant 1001\n  end\n\n  global :export_mutable do\n    @some_public_variable 42\n  end\n\n  # You can define multiple globals at once:\n  global do\n    @magic_number_a 99\n    @magic_number_b 12\n    @magic_number_c -5\n  end\nend\n```","ref":"Orb.html#global/2-examples"},{"type":"macro","title":"Orb.global/3","doc":"Declare WebAssembly globals.\n\n`mode` can be :readonly, :mutable, :export_readonly, or :export_mutable. The default is :mutable.","ref":"Orb.html#global/3"},{"type":"macro","title":"Examples - Orb.global/3","doc":"```elixir\ndefmodule GlobalExample do\n  use Orb\n\n  global do # :mutable by default\n    @some_internal_global 99\n  end\n\n  global :readonly do\n    @some_internal_constant 99\n  end\n\n  global :export_readonly do\n    @some_public_constant 1001\n  end\n\n  global :export_mutable do\n    @some_public_variable 42\n  end\n\n  # You can define multiple globals at once:\n  global do\n    @magic_number_a 99\n    @magic_number_b 12\n    @magic_number_c -5\n  end\nend\n```","ref":"Orb.html#global/3-examples"},{"type":"macro","title":"Orb.importw/2","doc":"Replaced by `Orb.Import.register/1`.","ref":"Orb.html#importw/2"},{"type":"macro","title":"Orb.include/1","doc":"Copy WebAssembly functions from one module into the current module.","ref":"Orb.html#include/1"},{"type":"macro","title":"Examples - Orb.include/1","doc":"```elixir\ndefmodule Math do\n  use Orb\n\n  defw square(n: I32), I32 do\n    n * n\n  end\nend\n\ndefmodule SomeOtherModule do\n  use Orb\n\n  Orb.include(Math)\n\n  defw magic(), I32 do\n    Math.square(3)\n  end\nend\n```","ref":"Orb.html#include/1-examples"},{"type":"macro","title":"Orb.set_func_prefix/1","doc":"Adds a prefix to all functions within this module. This namespacing helps avoid names from clashing.","ref":"Orb.html#set_func_prefix/1"},{"type":"macro","title":"Orb.snippet/3","doc":"Declare a snippet of Orb AST for reuse. Enables DSL, with additions from `mode`.","ref":"Orb.html#snippet/3"},{"type":"function","title":"Orb.to_wasm/1","doc":"Convert Orb AST into WebAssembly binary format.","ref":"Orb.html#to_wasm/1"},{"type":"function","title":"Orb.to_wat/1","doc":"Convert Orb AST into WebAssembly text format.","ref":"Orb.html#to_wat/1"},{"type":"macro","title":"Orb.types/1","doc":"","ref":"Orb.html#types/1"},{"type":"module","title":"Orb.Control","doc":"","ref":"Orb.Control.html"},{"type":"macro","title":"Orb.Control.block/3","doc":"Declare a block, useful for structured control flow.\n\n```elixir\ndefmodule Example do\n  use Orb\n\n  defw example do\n    Control.block Validate do\n      Validate.break(if: i < 0)\n\n      # Do something with i\n    end\n  end\nend\n```","ref":"Orb.Control.html#block/3"},{"type":"function","title":"Orb.Control.break/1","doc":"Break from a block.\n\n```elixir\ndefmodule Example do\n  use Orb\n\n  defw example do\n    Control.block Validate do\n      # Other code…\n\n      Control.break(Validate)\n    end\n  end\nend\n```","ref":"Orb.Control.html#break/1"},{"type":"function","title":"Orb.Control.break/2","doc":"Break from a block if a condition is true.\n\n```elixir\ndefmodule Example do\n  use Orb\n\n  defw example do\n    Control.block Validate do\n      Validate.break(if: i < 0)\n\n      # Do something with i\n    end\n  end\nend\n```","ref":"Orb.Control.html#break/2"},{"type":"function","title":"Orb.Control.return/0","doc":"","ref":"Orb.Control.html#return/0"},{"type":"behaviour","title":"Orb.CustomType","doc":"","ref":"Orb.CustomType.html"},{"type":"callback","title":"Orb.CustomType.load_instruction/0","doc":"","ref":"Orb.CustomType.html#c:load_instruction/0"},{"type":"function","title":"Orb.CustomType.resolve!/1","doc":"","ref":"Orb.CustomType.html#resolve!/1"},{"type":"callback","title":"Orb.CustomType.type_name/0","doc":"","ref":"Orb.CustomType.html#c:type_name/0"},{"type":"callback","title":"Orb.CustomType.wasm_type/0","doc":"","ref":"Orb.CustomType.html#c:wasm_type/0"},{"type":"type","title":"Orb.CustomType.wasm_primitive/0","doc":"","ref":"Orb.CustomType.html#t:wasm_primitive/0"},{"type":"module","title":"Orb.DSL","doc":"The main DSL which is imported automatically when you call `defw`.","ref":"Orb.DSL.html"},{"type":"function","title":"Orb.DSL.assert!/1","doc":"Asserts a condition that _must_ be true, otherwise traps with unreachable.","ref":"Orb.DSL.html#assert!/1"},{"type":"function","title":"Orb.DSL.const/1","doc":"Declare a constant string, which will be extracted to the top of the module, and its address substituted in place.","ref":"Orb.DSL.html#const/1"},{"type":"function","title":"Orb.DSL.do_match/3","doc":"","ref":"Orb.DSL.html#do_match/3"},{"type":"function","title":"Orb.DSL.do_snippet/2","doc":"","ref":"Orb.DSL.html#do_snippet/2"},{"type":"function","title":"Orb.DSL.export/2","doc":"","ref":"Orb.DSL.html#export/2"},{"type":"function","title":"Orb.DSL.f32/1","doc":"","ref":"Orb.DSL.html#f32/1"},{"type":"function","title":"Orb.DSL.f64/1","doc":"","ref":"Orb.DSL.html#f64/1"},{"type":"macro","title":"Orb.DSL.func/2","doc":"","ref":"Orb.DSL.html#func/2"},{"type":"macro","title":"Orb.DSL.func/3","doc":"","ref":"Orb.DSL.html#func/3"},{"type":"macro","title":"Orb.DSL.func/4","doc":"","ref":"Orb.DSL.html#func/4"},{"type":"function","title":"Orb.DSL.funcp/1","doc":"","ref":"Orb.DSL.html#funcp/1"},{"type":"macro","title":"Orb.DSL.funcp/2","doc":"","ref":"Orb.DSL.html#funcp/2"},{"type":"macro","title":"Orb.DSL.funcp/3","doc":"","ref":"Orb.DSL.html#funcp/3"},{"type":"macro","title":"Orb.DSL.funcp/4","doc":"","ref":"Orb.DSL.html#funcp/4"},{"type":"function","title":"Orb.DSL.i32/1","doc":"","ref":"Orb.DSL.html#i32/1"},{"type":"function","title":"Orb.DSL.i64/1","doc":"","ref":"Orb.DSL.html#i64/1"},{"type":"macro","title":"Orb.DSL.inline/1","doc":"Run code at compile-time.","ref":"Orb.DSL.html#inline/1"},{"type":"macro","title":"Orb.DSL.inline/2","doc":"Run Elixir for-comprehension at compile-time.","ref":"Orb.DSL.html#inline/2"},{"type":"macro","title":"Orb.DSL.loop/2","doc":"Declare a loop that iterates through an Orb.Iterator or Elixir.Range.\n\n```elixir\nsum = 0\nloop i <- 1..10 do\n  sum = sum + i\nend\n```\n\n```elixir\nsum = 0\nloop _ <- 1..10 do\n  sum = sum + 1\nend\n```\n\n```elixir\nsum = 0\nloop char <- alphabet do\n  sum = sum + 1\nend\n```\n\n```elixir\nsum = 0\nloop char <- alphabet do\n  sum = sum + char\nend\n```\n\n```elixir\nsum = 0\nloop _ <- alphabet do\n  sum = sum + 1\nend\n```","ref":"Orb.DSL.html#loop/2"},{"type":"macro","title":"Orb.DSL.loop/3","doc":"Declare a loop.","ref":"Orb.DSL.html#loop/3"},{"type":"function","title":"Orb.DSL.mut!/1","doc":"","ref":"Orb.DSL.html#mut!/1"},{"type":"function","title":"Orb.DSL.nop/0","doc":"A no-op instruction.\n\nSee https://en.wikipedia.org/wiki/NOP_(code)","ref":"Orb.DSL.html#nop/0"},{"type":"function","title":"Orb.DSL.return/0","doc":"Return from a function. You may wish to `push/1` values before returning.","ref":"Orb.DSL.html#return/0"},{"type":"function","title":"Orb.DSL.return/1","doc":"Return from a function if a condition is true.","ref":"Orb.DSL.html#return/1"},{"type":"function","title":"Orb.DSL.return/2","doc":"Return from a function with the provided value only if a condition is true.","ref":"Orb.DSL.html#return/2"},{"type":"function","title":"Orb.DSL.unreachable!/0","doc":"Denote a point in code that should not be reachable. Traps.\n\nUseful for exhaustive conditionals or code you know will not execute.","ref":"Orb.DSL.html#unreachable!/0"},{"type":"macro","title":"Orb.DSL.wasm/2","doc":"Used to bring back wasm-mode when inside `inline/1`.","ref":"Orb.DSL.html#wasm/2"},{"type":"module","title":"Orb.DSL.Defw","doc":"Macros to define WebAssembly functions. Similar to Elixir’s `def` and Nx’s `defn`.","ref":"Orb.DSL.Defw.html"},{"type":"macro","title":"Orb.DSL.Defw.defw/2","doc":"","ref":"Orb.DSL.Defw.html#defw/2"},{"type":"macro","title":"Orb.DSL.Defw.defw/3","doc":"","ref":"Orb.DSL.Defw.html#defw/3"},{"type":"macro","title":"Orb.DSL.Defw.defw/4","doc":"","ref":"Orb.DSL.Defw.html#defw/4"},{"type":"macro","title":"Orb.DSL.Defw.defwp/2","doc":"","ref":"Orb.DSL.Defw.html#defwp/2"},{"type":"macro","title":"Orb.DSL.Defw.defwp/3","doc":"","ref":"Orb.DSL.Defw.html#defwp/3"},{"type":"macro","title":"Orb.DSL.Defw.defwp/4","doc":"","ref":"Orb.DSL.Defw.html#defwp/4"},{"type":"macro","title":"Orb.DSL.Defw.wasm_mode/1","doc":"","ref":"Orb.DSL.Defw.html#wasm_mode/1"},{"type":"module","title":"Orb.Global.DSL","doc":"Adds @global_name support","ref":"Orb.Global.DSL.html"},{"type":"macro","title":"Orb.Global.DSL.@/1","doc":"","ref":"Orb.Global.DSL.html#@/1"},{"type":"module","title":"Orb.I32","doc":"Type for 32-bit integer.","ref":"Orb.I32.html"},{"type":"function","title":"Orb.I32.add/2","doc":"","ref":"Orb.I32.html#add/2"},{"type":"function","title":"Orb.I32.band/2","doc":"","ref":"Orb.I32.html#band/2"},{"type":"function","title":"Orb.I32.clz/1","doc":"","ref":"Orb.I32.html#clz/1"},{"type":"macro","title":"Orb.I32.cond/1","doc":"","ref":"Orb.I32.html#cond/1"},{"type":"function","title":"Orb.I32.const/1","doc":"","ref":"Orb.I32.html#const/1"},{"type":"function","title":"Orb.I32.ctz/1","doc":"","ref":"Orb.I32.html#ctz/1"},{"type":"function","title":"Orb.I32.div_s/2","doc":"","ref":"Orb.I32.html#div_s/2"},{"type":"function","title":"Orb.I32.div_u/2","doc":"","ref":"Orb.I32.html#div_u/2"},{"type":"macro","title":"Orb.I32.enum/2","doc":"","ref":"Orb.I32.html#enum/2"},{"type":"function","title":"Orb.I32.eq/2","doc":"","ref":"Orb.I32.html#eq/2"},{"type":"function","title":"Orb.I32.eqz/1","doc":"","ref":"Orb.I32.html#eqz/1"},{"type":"macro","title":"Orb.I32.export_enum/2","doc":"","ref":"Orb.I32.html#export_enum/2"},{"type":"macro","title":"Orb.I32.export_global/2","doc":"","ref":"Orb.I32.html#export_global/2"},{"type":"function","title":"Orb.I32.from_4_byte_ascii/1","doc":"Converts an ASCII little-endian 4-byte string to an 32-bit integer.","ref":"Orb.I32.html#from_4_byte_ascii/1"},{"type":"function","title":"Orb.I32.ge_s/2","doc":"","ref":"Orb.I32.html#ge_s/2"},{"type":"function","title":"Orb.I32.ge_u/2","doc":"","ref":"Orb.I32.html#ge_u/2"},{"type":"macro","title":"Orb.I32.global/2","doc":"","ref":"Orb.I32.html#global/2"},{"type":"function","title":"Orb.I32.gt_s/2","doc":"","ref":"Orb.I32.html#gt_s/2"},{"type":"function","title":"Orb.I32.gt_u/2","doc":"","ref":"Orb.I32.html#gt_u/2"},{"type":"function","title":"Orb.I32.in?/2","doc":"","ref":"Orb.I32.html#in?/2"},{"type":"function","title":"Orb.I32.le_s/2","doc":"","ref":"Orb.I32.html#le_s/2"},{"type":"function","title":"Orb.I32.le_u/2","doc":"","ref":"Orb.I32.html#le_u/2"},{"type":"function","title":"Orb.I32.lt_s/2","doc":"","ref":"Orb.I32.html#lt_s/2"},{"type":"function","title":"Orb.I32.lt_u/2","doc":"","ref":"Orb.I32.html#lt_u/2"},{"type":"macro","title":"Orb.I32.match/2","doc":"","ref":"Orb.I32.html#match/2"},{"type":"function","title":"Orb.I32.mul/2","doc":"","ref":"Orb.I32.html#mul/2"},{"type":"function","title":"Orb.I32.ne/2","doc":"","ref":"Orb.I32.html#ne/2"},{"type":"function","title":"Orb.I32.or/2","doc":"","ref":"Orb.I32.html#or/2"},{"type":"function","title":"Orb.I32.popcnt/1","doc":"","ref":"Orb.I32.html#popcnt/1"},{"type":"function","title":"Orb.I32.reinterpret_f32/1","doc":"","ref":"Orb.I32.html#reinterpret_f32/1"},{"type":"function","title":"Orb.I32.rem_s/2","doc":"","ref":"Orb.I32.html#rem_s/2"},{"type":"function","title":"Orb.I32.rem_u/2","doc":"","ref":"Orb.I32.html#rem_u/2"},{"type":"function","title":"Orb.I32.rotl/2","doc":"","ref":"Orb.I32.html#rotl/2"},{"type":"function","title":"Orb.I32.rotr/2","doc":"","ref":"Orb.I32.html#rotr/2"},{"type":"function","title":"Orb.I32.shl/2","doc":"","ref":"Orb.I32.html#shl/2"},{"type":"function","title":"Orb.I32.shr_s/2","doc":"","ref":"Orb.I32.html#shr_s/2"},{"type":"function","title":"Orb.I32.shr_u/2","doc":"","ref":"Orb.I32.html#shr_u/2"},{"type":"function","title":"Orb.I32.sub/2","doc":"","ref":"Orb.I32.html#sub/2"},{"type":"function","title":"Orb.I32.sum!/1","doc":"","ref":"Orb.I32.html#sum!/1"},{"type":"function","title":"Orb.I32.trunc_f32_s/1","doc":"","ref":"Orb.I32.html#trunc_f32_s/1"},{"type":"function","title":"Orb.I32.trunc_f32_u/1","doc":"","ref":"Orb.I32.html#trunc_f32_u/1"},{"type":"function","title":"Orb.I32.trunc_f64_s/1","doc":"","ref":"Orb.I32.html#trunc_f64_s/1"},{"type":"function","title":"Orb.I32.trunc_f64_u/1","doc":"","ref":"Orb.I32.html#trunc_f64_u/1"},{"type":"function","title":"Orb.I32.wrap_i64/1","doc":"","ref":"Orb.I32.html#wrap_i64/1"},{"type":"function","title":"Orb.I32.xor/2","doc":"","ref":"Orb.I32.html#xor/2"},{"type":"module","title":"Orb.I32.DSL","doc":"The operators that work consistently signed or unsigned.\n\nSigned specific: `Orb.S32.DSL`\n\nUnsigned specific: `Orb.U32.DSL`","ref":"Orb.I32.DSL.html"},{"type":"function","title":"Orb.I32.DSL.&&&/2","doc":"","ref":"Orb.I32.DSL.html#&&&/2"},{"type":"function","title":"Orb.I32.DSL.*/2","doc":"","ref":"Orb.I32.DSL.html#*/2"},{"type":"function","title":"Orb.I32.DSL.+/2","doc":"","ref":"Orb.I32.DSL.html#+/2"},{"type":"function","title":"Orb.I32.DSL.-/2","doc":"","ref":"Orb.I32.DSL.html#-/2"},{"type":"function","title":"Orb.I32.DSL.!=/2","doc":"","ref":"Orb.I32.DSL.html#!=/2"},{"type":"function","title":"Orb.I32.DSL.!==/2","doc":"","ref":"Orb.I32.DSL.html#!==/2"},{"type":"function","title":"Orb.I32.DSL.<<</2","doc":"","ref":"Orb.I32.DSL.html#%3C%3C%3C/2"},{"type":"function","title":"Orb.I32.DSL.==/2","doc":"","ref":"Orb.I32.DSL.html#==/2"},{"type":"function","title":"Orb.I32.DSL.===/2","doc":"","ref":"Orb.I32.DSL.html#===/2"},{"type":"function","title":"Orb.I32.DSL.not/1","doc":"","ref":"Orb.I32.DSL.html#not/1"},{"type":"function","title":"Orb.I32.DSL.or/2","doc":"","ref":"Orb.I32.DSL.html#or/2"},{"type":"function","title":"Orb.I32.DSL.|||/2","doc":"","ref":"Orb.I32.DSL.html#%7C%7C%7C/2"},{"type":"module","title":"Orb.I32.U16","doc":"Type for unsigned 32-bit integer interpreted as unsigned 16-bit (double byte).","ref":"Orb.I32.U16.html"},{"type":"module","title":"Orb.I32.U8","doc":"Type for unsigned 32-bit integer interpreted as an unsigned 8-bit byte.","ref":"Orb.I32.U8.html"},{"type":"module","title":"Orb.I32.U8.UnsafePointer","doc":"Custom `Orb.CustomType` for pointer to 8-bit integer (byte) in memory.","ref":"Orb.I32.U8.UnsafePointer.html"},{"type":"function","title":"Orb.I32.U8.UnsafePointer.memory_alignment/0","doc":"","ref":"Orb.I32.U8.UnsafePointer.html#memory_alignment/0"},{"type":"module","title":"Orb.I32.UnsafePointer","doc":"Custom `Orb.CustomType` for pointer to a 32-bit integer in memory.\n\nIt is aligned to 4 bytes or 32-bits, so you can only load/store at memory offsets divisible by 4.","ref":"Orb.I32.UnsafePointer.html"},{"type":"function","title":"Orb.I32.UnsafePointer.memory_alignment/0","doc":"","ref":"Orb.I32.UnsafePointer.html#memory_alignment/0"},{"type":"module","title":"Orb.I64","doc":"Type for 64-bit integer.","ref":"Orb.I64.html"},{"type":"function","title":"Orb.I64.add/2","doc":"","ref":"Orb.I64.html#add/2"},{"type":"function","title":"Orb.I64.band/2","doc":"","ref":"Orb.I64.html#band/2"},{"type":"function","title":"Orb.I64.clz/1","doc":"","ref":"Orb.I64.html#clz/1"},{"type":"function","title":"Orb.I64.ctz/1","doc":"","ref":"Orb.I64.html#ctz/1"},{"type":"function","title":"Orb.I64.div_s/2","doc":"","ref":"Orb.I64.html#div_s/2"},{"type":"function","title":"Orb.I64.div_u/2","doc":"","ref":"Orb.I64.html#div_u/2"},{"type":"function","title":"Orb.I64.eq/2","doc":"","ref":"Orb.I64.html#eq/2"},{"type":"function","title":"Orb.I64.eqz/1","doc":"","ref":"Orb.I64.html#eqz/1"},{"type":"function","title":"Orb.I64.extend_i32_s/1","doc":"Convert signed i32 to signed i64.","ref":"Orb.I64.html#extend_i32_s/1"},{"type":"function","title":"Orb.I64.extend_i32_u/1","doc":"Convert unsigned i32 to unsigned i64.","ref":"Orb.I64.html#extend_i32_u/1"},{"type":"function","title":"Orb.I64.ge_s/2","doc":"","ref":"Orb.I64.html#ge_s/2"},{"type":"function","title":"Orb.I64.ge_u/2","doc":"","ref":"Orb.I64.html#ge_u/2"},{"type":"function","title":"Orb.I64.gt_s/2","doc":"","ref":"Orb.I64.html#gt_s/2"},{"type":"function","title":"Orb.I64.gt_u/2","doc":"","ref":"Orb.I64.html#gt_u/2"},{"type":"function","title":"Orb.I64.le_s/2","doc":"","ref":"Orb.I64.html#le_s/2"},{"type":"function","title":"Orb.I64.le_u/2","doc":"","ref":"Orb.I64.html#le_u/2"},{"type":"function","title":"Orb.I64.lt_s/2","doc":"","ref":"Orb.I64.html#lt_s/2"},{"type":"function","title":"Orb.I64.lt_u/2","doc":"","ref":"Orb.I64.html#lt_u/2"},{"type":"function","title":"Orb.I64.mul/2","doc":"","ref":"Orb.I64.html#mul/2"},{"type":"function","title":"Orb.I64.ne/2","doc":"","ref":"Orb.I64.html#ne/2"},{"type":"function","title":"Orb.I64.or/2","doc":"","ref":"Orb.I64.html#or/2"},{"type":"function","title":"Orb.I64.popcnt/1","doc":"","ref":"Orb.I64.html#popcnt/1"},{"type":"function","title":"Orb.I64.reinterpret_f64/1","doc":"","ref":"Orb.I64.html#reinterpret_f64/1"},{"type":"function","title":"Orb.I64.rem_s/2","doc":"","ref":"Orb.I64.html#rem_s/2"},{"type":"function","title":"Orb.I64.rem_u/2","doc":"","ref":"Orb.I64.html#rem_u/2"},{"type":"function","title":"Orb.I64.rotl/2","doc":"","ref":"Orb.I64.html#rotl/2"},{"type":"function","title":"Orb.I64.rotr/2","doc":"","ref":"Orb.I64.html#rotr/2"},{"type":"function","title":"Orb.I64.shl/2","doc":"","ref":"Orb.I64.html#shl/2"},{"type":"function","title":"Orb.I64.shr_s/2","doc":"","ref":"Orb.I64.html#shr_s/2"},{"type":"function","title":"Orb.I64.shr_u/2","doc":"","ref":"Orb.I64.html#shr_u/2"},{"type":"function","title":"Orb.I64.sub/2","doc":"","ref":"Orb.I64.html#sub/2"},{"type":"function","title":"Orb.I64.trunc_f32_s/1","doc":"","ref":"Orb.I64.html#trunc_f32_s/1"},{"type":"function","title":"Orb.I64.trunc_f32_u/1","doc":"","ref":"Orb.I64.html#trunc_f32_u/1"},{"type":"function","title":"Orb.I64.trunc_f64_s/1","doc":"","ref":"Orb.I64.html#trunc_f64_s/1"},{"type":"function","title":"Orb.I64.trunc_f64_u/1","doc":"","ref":"Orb.I64.html#trunc_f64_u/1"},{"type":"function","title":"Orb.I64.xor/2","doc":"","ref":"Orb.I64.html#xor/2"},{"type":"module","title":"Orb.I64.DSL","doc":"The operators that work consistently signed or unsigned.\n\nSigned specific: `Orb.S64.DSL`\n\nUnsigned specific: `Orb.U64.DSL`","ref":"Orb.I64.DSL.html"},{"type":"function","title":"Orb.I64.DSL.&&&/2","doc":"","ref":"Orb.I64.DSL.html#&&&/2"},{"type":"function","title":"Orb.I64.DSL.*/2","doc":"","ref":"Orb.I64.DSL.html#*/2"},{"type":"function","title":"Orb.I64.DSL.+/2","doc":"","ref":"Orb.I64.DSL.html#+/2"},{"type":"function","title":"Orb.I64.DSL.-/2","doc":"","ref":"Orb.I64.DSL.html#-/2"},{"type":"function","title":"Orb.I64.DSL.!=/2","doc":"","ref":"Orb.I64.DSL.html#!=/2"},{"type":"function","title":"Orb.I64.DSL.!==/2","doc":"","ref":"Orb.I64.DSL.html#!==/2"},{"type":"function","title":"Orb.I64.DSL.<<</2","doc":"","ref":"Orb.I64.DSL.html#%3C%3C%3C/2"},{"type":"function","title":"Orb.I64.DSL.==/2","doc":"","ref":"Orb.I64.DSL.html#==/2"},{"type":"function","title":"Orb.I64.DSL.===/2","doc":"","ref":"Orb.I64.DSL.html#===/2"},{"type":"function","title":"Orb.I64.DSL.not/1","doc":"","ref":"Orb.I64.DSL.html#not/1"},{"type":"function","title":"Orb.I64.DSL.or/2","doc":"","ref":"Orb.I64.DSL.html#or/2"},{"type":"function","title":"Orb.I64.DSL.|||/2","doc":"","ref":"Orb.I64.DSL.html#%7C%7C%7C/2"},{"type":"module","title":"Orb.I64.Signed.DSL","doc":"Signed 32-bit integer operators.","ref":"Orb.I64.Signed.DSL.html"},{"type":"function","title":"Orb.I64.Signed.DSL.//2","doc":"","ref":"Orb.I64.Signed.DSL.html#//2"},{"type":"function","title":"Orb.I64.Signed.DSL.</2","doc":"","ref":"Orb.I64.Signed.DSL.html#%3C/2"},{"type":"function","title":"Orb.I64.Signed.DSL.<=/2","doc":"","ref":"Orb.I64.Signed.DSL.html#%3C=/2"},{"type":"function","title":"Orb.I64.Signed.DSL.>/2","doc":"","ref":"Orb.I64.Signed.DSL.html#%3E/2"},{"type":"function","title":"Orb.I64.Signed.DSL.>=/2","doc":"","ref":"Orb.I64.Signed.DSL.html#%3E=/2"},{"type":"function","title":"Orb.I64.Signed.DSL.>>>/2","doc":"","ref":"Orb.I64.Signed.DSL.html#%3E%3E%3E/2"},{"type":"module","title":"Orb.Import","doc":"Define an interface to be imported into WebAssembly.\n\nEach function declaration is turned into an `(import)` WebAssembly instruction.\n\n```elixir\ndefmodule Log do\n  use Orb.Import, name: :log\n\n  defw logi32(value: I32)\n  defw logi64(value: I64)\nend\n\ndefmodule Time do\n  use Orb.Import, name: :time\n\n  defw get_unix_time(), I64\nend\n\ndefmodule Example do\n  use Orb\n\n  Orb.Import.register(Log)\n  Orb.Import.register(Time)\n\n  defw example() do\n    Log.logi32(99)\n\n    Log.logi64(Time.get_unix_time())\n  end\nend\n```","ref":"Orb.Import.html"},{"type":"macro","title":"Orb.Import.register/1","doc":"Register an import `module` under `namespace`.\n\nThe `module` is defined via `use Orb.Import`. The `namespace` is the local name the import is registered under.\n\nFor each function defined within `module` an `(import)` instruction is defined in the outputted WebAssembly:\n\n```wasm\n(module $example\n  (import \"namespace\" \"function1\" (func $function1 (param $a i32) (result i32)))\n  (import \"namespace\" \"function2\" (func $function2 (param $a i32) (result i32)))\n  (import \"namespace\" \"function3\" (func $function3 (param $a i32) (result i32)))\n)\n```","ref":"Orb.Import.html#register/1"},{"type":"module","title":"Orb.Import.DSL","doc":"DSL for declaring within Elixir module that get converted to `(import …)` expressions in WebAssembly.","ref":"Orb.Import.DSL.html"},{"type":"macro","title":"Orb.Import.DSL.defw/2","doc":"Declare the name and signature of function that will be imported.\n\n```elixir\ndefmodule Log do\n  use Orb.Import, name: :log\n\n  defw(int32(a: I32))\n  defw(int64(a: I64))\n  defw(two_int32(a: I32, b: I32))\nend\n```","ref":"Orb.Import.DSL.html#defw/2"},{"type":"behaviour","title":"Orb.Iterator","doc":"","ref":"Orb.Iterator.html"},{"type":"callback","title":"Orb.Iterator.next/1","doc":"Mutates the iterator advancing it to the next element.","ref":"Orb.Iterator.html#c:next/1"},{"type":"callback","title":"Orb.Iterator.valid?/1","doc":"Calculates whether the iterator is done or not.","ref":"Orb.Iterator.html#c:valid?/1"},{"type":"callback","title":"Orb.Iterator.value/1","doc":"Extracts or calculates the current element value.","ref":"Orb.Iterator.html#c:value/1"},{"type":"callback","title":"Orb.Iterator.value_type/0","doc":"The type for iterated elements.","ref":"Orb.Iterator.html#c:value_type/0"},{"type":"type","title":"Orb.Iterator.var_ref/0","doc":"","ref":"Orb.Iterator.html#t:var_ref/0"},{"type":"module","title":"Orb.Loop.Branch","doc":"","ref":"Orb.Loop.Branch.html"},{"type":"module","title":"Orb.Memory","doc":"Work with memory: load, store, declare pages & initial data.","ref":"Orb.Memory.html"},{"type":"macro","title":"Orb.Memory.export/1","doc":"","ref":"Orb.Memory.html#export/1"},{"type":"function","title":"Orb.Memory.grow!/1","doc":"Increases the size of the memory instance by a specified number of pages. Each page is 64KiB.\n\nNote: only `0` and `1` are supported in today’s runtimes.\n\nIt returns the previous size of memory in pages if the operation was successful, or returns -1 if the operation failed.\n\nhttps://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Memory/Grow\n\n```elixir\nMemory.grow!(1)\n```","ref":"Orb.Memory.html#grow!/1"},{"type":"macro","title":"Orb.Memory.import/2","doc":"","ref":"Orb.Memory.html#import/2"},{"type":"macro","title":"Orb.Memory.initial_data!/2","doc":"Initializes data in memory. In Wat is `(data …)`","ref":"Orb.Memory.html#initial_data!/2"},{"type":"function","title":"Orb.Memory.load!/3","doc":"Load value of `type` from memory `address`.\n\n```elixir\nMemory.load!(I32, 0x100)\nMemory.load!(I32.U8, 0x100)\n```","ref":"Orb.Memory.html#load!/3"},{"type":"function","title":"Examples - Orb.Memory.load!/3","doc":"iex> use Orb\n    iex> Memory.load!(I32, 0x100) |> Orb.to_wat()\n    \"(i32.load (i32.const 256))\"\n    iex> Memory.load!(I32.U8, 0x100) |> Orb.to_wat()\n    \"(i32.load8_u (i32.const 256))\"\n    iex> Memory.load!(I32, 0x100, align: 2) |> Orb.to_wat()\n    \"(i32.load align=2 (i32.const 256))\"\n    iex> Memory.load!(I32, 0x100, align: 4) |> Orb.to_wat()\n    \"(i32.load align=4 (i32.const 256))\"\n\n    iex> use Orb\n    iex> Memory.load!(I32, 0x100, align: 3)\n    ** (ArgumentError) malformed alignment 3\n\n    iex> use Orb\n    iex> Memory.load!(I32, 0x100, align: 8)\n    ** (ArgumentError) alignment 8 must not be larger than natural 4\n\n    iex> use Orb\n    iex> Memory.load!(I32.U8, 0x100, align: 2)\n    ** (ArgumentError) alignment 2 must not be larger than natural 1","ref":"Orb.Memory.html#load!/3-examples"},{"type":"function","title":"Orb.Memory.page_byte_size/0","doc":"","ref":"Orb.Memory.html#page_byte_size/0"},{"type":"macro","title":"Orb.Memory.pages/1","doc":"Declare how many 64Kib pages of memory your module needs.\n\nCan be called multiple times, with each summed up.\n\nReturns the previous page count, which can be used as a start offset.","ref":"Orb.Memory.html#pages/1"},{"type":"function","title":"Orb.Memory.size/0","doc":"Returns the number of pages the memory instance currently has. Each page is 64KiB.\n\nTo increase the number of pages call `grow!/1`.\n\nhttps://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Memory/Size\n\n```elixir\nMemory.size()\n```","ref":"Orb.Memory.html#size/0"},{"type":"function","title":"Orb.Memory.store!/4","doc":"Store `value` of `type` at memory `address`.\n\n```elixir\nMemory.store!(I32, 0x100, 42)\nMemory.store!(I32.U8, 0x100, ?a)\n```","ref":"Orb.Memory.html#store!/4"},{"type":"function","title":"Examples - Orb.Memory.store!/4","doc":"iex> use Orb\n    iex> Memory.store!(I32, 0x100, 42) |> Orb.to_wat()\n    \"(i32.store (i32.const 256) (i32.const 42))\"\n    iex> Memory.store!(I32.U8, 0x100, ?a) |> Orb.to_wat()\n    \"(i32.store8 (i32.const 256) (i32.const 97))\"\n    iex> Memory.store!(I32, 0x100, 42, align: 2) |> Orb.to_wat()\n    \"(i32.store align=2 (i32.const 256) (i32.const 42))\"\n    iex> Memory.store!(I32, 0x100, 42, align: 4) |> Orb.to_wat()\n    \"(i32.store align=4 (i32.const 256) (i32.const 42))\"\n\n    iex> use Orb\n    iex> Memory.store!(I32, 0x100, 42, align: 3)\n    ** (ArgumentError) malformed alignment 3\n\n    iex> use Orb\n    iex> Memory.store!(I32, 0x100, 42, align: 8)\n    ** (ArgumentError) alignment 8 must not be larger than natural 4\n\n    iex> use Orb\n    iex> Memory.store!(I32.U8, 0x100, 42, align: 2)\n    ** (ArgumentError) alignment 2 must not be larger than natural 1","ref":"Orb.Memory.html#store!/4-examples"},{"type":"function","title":"Orb.Memory.swap!/4","doc":"A convenience for swapping two values of `type` at addresses `address_a` and `address_b` in memory.\n\nBubble sort is an example of an algorithm which uses a swap.","ref":"Orb.Memory.html#swap!/4"},{"type":"module","title":"Orb.Memory.Slice","doc":"Similar to [Rust’s slice](https://doc.rust-lang.org/1.77.0/std/primitive.slice.html) but across all memory.","ref":"Orb.Memory.Slice.html"},{"type":"function","title":"Orb.Memory.Slice.drop_first_byte/1","doc":"","ref":"Orb.Memory.Slice.html#drop_first_byte/1"},{"type":"function","title":"Orb.Memory.Slice.from/2","doc":"","ref":"Orb.Memory.Slice.html#from/2"},{"type":"function","title":"Orb.Memory.Slice.get_byte_length/1","doc":"","ref":"Orb.Memory.Slice.html#get_byte_length/1"},{"type":"function","title":"Orb.Memory.Slice.get_byte_offset/1","doc":"","ref":"Orb.Memory.Slice.html#get_byte_offset/1"},{"type":"module","title":"Orb.MutRef","doc":"Use `Orb.DSL.mut!/1` to get a mutable reference to a global or local.","ref":"Orb.MutRef.html"},{"type":"function","title":"Orb.MutRef.from/1","doc":"","ref":"Orb.MutRef.html#from/1"},{"type":"function","title":"Orb.MutRef.store/2","doc":"","ref":"Orb.MutRef.html#store/2"},{"type":"module","title":"Orb.Numeric.Add","doc":"WebAssembly numeric `add` instruction:\n- `i32.add`\n- `i64.add`\n- `f32.add`\n- `f64.add`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Addition)","ref":"Orb.Numeric.Add.html"},{"type":"function","title":"Orb.Numeric.Add.optimized/3","doc":"","ref":"Orb.Numeric.Add.html#optimized/3"},{"type":"module","title":"Orb.Numeric.DSL","doc":"Operators that work with all number types.","ref":"Orb.Numeric.DSL.html"},{"type":"function","title":"Orb.Numeric.DSL.&&&/2","doc":"Bitwise AND operation for i32 or i64 integers.","ref":"Orb.Numeric.DSL.html#&&&/2"},{"type":"function","title":"Examples - Orb.Numeric.DSL.&&&/2","doc":"use Orb\n\n    defw bitwise_and_example(a: I32, b: I32), I32 do\n      a &&& b\n    end","ref":"Orb.Numeric.DSL.html#&&&/2-examples"},{"type":"function","title":"Orb.Numeric.DSL.*/2","doc":"","ref":"Orb.Numeric.DSL.html#*/2"},{"type":"function","title":"Orb.Numeric.DSL.+/2","doc":"","ref":"Orb.Numeric.DSL.html#+/2"},{"type":"function","title":"Orb.Numeric.DSL.-/2","doc":"","ref":"Orb.Numeric.DSL.html#-/2"},{"type":"function","title":"Orb.Numeric.DSL.//2","doc":"","ref":"Orb.Numeric.DSL.html#//2"},{"type":"function","title":"Orb.Numeric.DSL.!=/2","doc":"","ref":"Orb.Numeric.DSL.html#!=/2"},{"type":"function","title":"Orb.Numeric.DSL.!==/2","doc":"","ref":"Orb.Numeric.DSL.html#!==/2"},{"type":"function","title":"Orb.Numeric.DSL.</2","doc":"","ref":"Orb.Numeric.DSL.html#%3C/2"},{"type":"function","title":"Orb.Numeric.DSL.<<</2","doc":"Left shift operation for i32 or i64 integers.","ref":"Orb.Numeric.DSL.html#%3C%3C%3C/2"},{"type":"function","title":"Examples - Orb.Numeric.DSL.<<</2","doc":"use Orb\n\n    defw left_shift_by_4(a: I32), I32 do\n      a <<< 4\n    end","ref":"Orb.Numeric.DSL.html#%3C%3C%3C/2-examples"},{"type":"function","title":"Orb.Numeric.DSL.<=/2","doc":"","ref":"Orb.Numeric.DSL.html#%3C=/2"},{"type":"function","title":"Orb.Numeric.DSL.==/2","doc":"","ref":"Orb.Numeric.DSL.html#==/2"},{"type":"function","title":"Orb.Numeric.DSL.===/2","doc":"","ref":"Orb.Numeric.DSL.html#===/2"},{"type":"function","title":"Orb.Numeric.DSL.>/2","doc":"","ref":"Orb.Numeric.DSL.html#%3E/2"},{"type":"function","title":"Orb.Numeric.DSL.>=/2","doc":"","ref":"Orb.Numeric.DSL.html#%3E=/2"},{"type":"function","title":"Orb.Numeric.DSL.and/2","doc":"Logical AND operation for i32 or i64 integers. Returns i32.\n\nZero is considered false, all other numbers are considered true.\n\nIt does not support short-circuiting.","ref":"Orb.Numeric.DSL.html#and/2"},{"type":"function","title":"Examples - Orb.Numeric.DSL.and/2","doc":"use Orb\n\n    defw and_example(a: I32, b: I32), I32 do\n      a and b\n    end","ref":"Orb.Numeric.DSL.html#and/2-examples"},{"type":"function","title":"Orb.Numeric.DSL.not/1","doc":"Logical NOT operation for i32 or i64 integers. Returns i32.\n\nZero is considered false, all other numbers are considered true.\n\nIt does not support short-circuiting.","ref":"Orb.Numeric.DSL.html#not/1"},{"type":"function","title":"Examples - Orb.Numeric.DSL.not/1","doc":"use Orb\n\n    defw not_example(a: I32), I32 do\n      not a\n    end","ref":"Orb.Numeric.DSL.html#not/1-examples"},{"type":"function","title":"Orb.Numeric.DSL.or/2","doc":"Logical OR operation for i32 or i64 integers. Returns i32.\n\nZero is considered false, all other numbers are considered true.\n\nIt does not support short-circuiting.","ref":"Orb.Numeric.DSL.html#or/2"},{"type":"function","title":"Examples - Orb.Numeric.DSL.or/2","doc":"use Orb\n\n    defw or_example(a: I32, b: I32), I32 do\n      a or b\n    end","ref":"Orb.Numeric.DSL.html#or/2-examples"},{"type":"function","title":"Orb.Numeric.DSL.|||/2","doc":"Bitwise OR operation for i32 or i64 integers.","ref":"Orb.Numeric.DSL.html#%7C%7C%7C/2"},{"type":"function","title":"Examples - Orb.Numeric.DSL.|||/2","doc":"use Orb\n\n    defw bitwise_or_example(a: I32, b: I32), I32 do\n      a ||| b\n    end","ref":"Orb.Numeric.DSL.html#%7C%7C%7C/2-examples"},{"type":"module","title":"Orb.Numeric.Multiply","doc":"WebAssembly numeric `mul` instruction:\n- `i32.mul`\n- `i64.mul`\n- `f32.mul`\n- `f64.mul`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Multiplication)","ref":"Orb.Numeric.Multiply.html"},{"type":"function","title":"Orb.Numeric.Multiply.optimized/3","doc":"","ref":"Orb.Numeric.Multiply.html#optimized/3"},{"type":"module","title":"Orb.Numeric.NotEqual","doc":"WebAssembly numeric `ne` instruction:\n- `i32.ne`\n- `i64.ne`\n- `f32.ne`\n- `f64.ne`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Not_equal)","ref":"Orb.Numeric.NotEqual.html"},{"type":"function","title":"Orb.Numeric.NotEqual.optimized/3","doc":"","ref":"Orb.Numeric.NotEqual.html#optimized/3"},{"type":"module","title":"Orb.Numeric.Subtract","doc":"WebAssembly numeric `sub` instruction:\n- `i32.sub`\n- `i64.sub`\n- `f32.sub`\n- `f64.sub`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Subtraction)","ref":"Orb.Numeric.Subtract.html"},{"type":"function","title":"Orb.Numeric.Subtract.optimized/3","doc":"","ref":"Orb.Numeric.Subtract.html#optimized/3"},{"type":"module","title":"Orb.S32.DSL","doc":"Signed 32-bit integer operators.","ref":"Orb.S32.DSL.html"},{"type":"function","title":"Orb.S32.DSL.//2","doc":"","ref":"Orb.S32.DSL.html#//2"},{"type":"function","title":"Orb.S32.DSL.</2","doc":"","ref":"Orb.S32.DSL.html#%3C/2"},{"type":"function","title":"Orb.S32.DSL.<=/2","doc":"","ref":"Orb.S32.DSL.html#%3C=/2"},{"type":"function","title":"Orb.S32.DSL.>/2","doc":"","ref":"Orb.S32.DSL.html#%3E/2"},{"type":"function","title":"Orb.S32.DSL.>=/2","doc":"","ref":"Orb.S32.DSL.html#%3E=/2"},{"type":"function","title":"Orb.S32.DSL.>>>/2","doc":"","ref":"Orb.S32.DSL.html#%3E%3E%3E/2"},{"type":"module","title":"Orb.Stack","doc":"Functions for working with the WebAssembly function stack.\n\nParticularly useful for inlined algorithms which can push a value to the stack and later pop it, avoiding the need to declare another local.","ref":"Orb.Stack.html"},{"type":"function","title":"Orb.Stack.drop/1","doc":"Executes the passed instruction and immediately drops its result(s) from the stack, effectively ignoring the resulting value.","ref":"Orb.Stack.html#drop/1"},{"type":"function","title":"Orb.Stack.pop/1","doc":"Pops the last value from the stack, useful for assigning it to a local.","ref":"Orb.Stack.html#pop/1"},{"type":"function","title":"Orb.Stack.push/1","doc":"Pushes a value onto the current stack.","ref":"Orb.Stack.html#push/1"},{"type":"macro","title":"Orb.Stack.push/2","doc":"Push value then run the block. Useful for when you mutate a variable but want its previous value.","ref":"Orb.Stack.html#push/2"},{"type":"module","title":"Orb.Str","doc":"A string slice. Represented under the hood as a `{:i32, :i32}` tuple, the first element being the base address, and the second element the string length.","ref":"Orb.Str.html"},{"type":"function","title":"Orb.Str.empty/0","doc":"An empty string, represented by a string slice from offset zero and of zero length.\n\nThe ol’ billion-dollar mistake: null.","ref":"Orb.Str.html#empty/0"},{"type":"function","title":"Orb.Str.get_base_address/1","doc":"","ref":"Orb.Str.html#get_base_address/1"},{"type":"function","title":"Orb.Str.to_slice/1","doc":"","ref":"Orb.Str.html#to_slice/1"},{"type":"module","title":"Orb.Str.Slice","doc":"A string slice represented as a i64, a compromise for when storing strings as globals (Globals cannot be `(i32 i32)`, only single primitives like `i32` or `i64`).","ref":"Orb.Str.Slice.html"},{"type":"function","title":"Orb.Str.Slice.to_str/1","doc":"","ref":"Orb.Str.Slice.html#to_str/1"},{"type":"module","title":"Orb.Table","doc":"","ref":"Orb.Table.html"},{"type":"macro","title":"Orb.Table.allocate/1","doc":"","ref":"Orb.Table.html#allocate/1"},{"type":"function","title":"Orb.Table.call/1","doc":"","ref":"Orb.Table.html#call/1"},{"type":"function","title":"Orb.Table.call/2","doc":"","ref":"Orb.Table.html#call/2"},{"type":"function","title":"Orb.Table.call/3","doc":"","ref":"Orb.Table.html#call/3"},{"type":"function","title":"Orb.Table.call/4","doc":"","ref":"Orb.Table.html#call/4"},{"type":"function","title":"Orb.Table.call/5","doc":"","ref":"Orb.Table.html#call/5"},{"type":"function","title":"Orb.Table.call_indirect/3","doc":"","ref":"Orb.Table.html#call_indirect/3"},{"type":"function","title":"Orb.Table.call_indirect/4","doc":"","ref":"Orb.Table.html#call_indirect/4"},{"type":"function","title":"Orb.Table.call_indirect/5","doc":"","ref":"Orb.Table.html#call_indirect/5"},{"type":"function","title":"Orb.Table.call_indirect/6","doc":"","ref":"Orb.Table.html#call_indirect/6"},{"type":"function","title":"Orb.Table.call_indirect/7","doc":"","ref":"Orb.Table.html#call_indirect/7"},{"type":"function","title":"Orb.Table.do_elem/4","doc":"","ref":"Orb.Table.html#do_elem/4"},{"type":"macro","title":"Orb.Table.elem!/3","doc":"","ref":"Orb.Table.html#elem!/3"},{"type":"macro","title":"Orb.Table.fetch!/1","doc":"","ref":"Orb.Table.html#fetch!/1"},{"type":"macro","title":"Orb.Table.lookup!/1","doc":"","ref":"Orb.Table.html#lookup!/1"},{"type":"macro","title":"Orb.Table.lookup!/2","doc":"","ref":"Orb.Table.html#lookup!/2"},{"type":"behaviour","title":"Orb.Table.Type","doc":"","ref":"Orb.Table.Type.html"},{"type":"callback","title":"Orb.Table.Type.table_func_keys/0","doc":"","ref":"Orb.Table.Type.html#c:table_func_keys/0"},{"type":"protocol","title":"Orb.ToWasm","doc":"","ref":"Orb.ToWasm.html"},{"type":"function","title":"Orb.ToWasm.to_wasm/2","doc":"Protocol for converting to WebAssembly binary format (.wasm)\n\nThere’s a helpful table / cheatsheet of instruction opcodes here: https://pengowray.github.io/wasm-ops/","ref":"Orb.ToWasm.html#to_wasm/2"},{"type":"type","title":"Orb.ToWasm.t/0","doc":"All the types that implement this protocol.","ref":"Orb.ToWasm.html#t:t/0"},{"type":"module","title":"Orb.ToWasm.Context","doc":"","ref":"Orb.ToWasm.Context.html"},{"type":"function","title":"Orb.ToWasm.Context.fetch_func_index!/2","doc":"","ref":"Orb.ToWasm.Context.html#fetch_func_index!/2"},{"type":"function","title":"Orb.ToWasm.Context.fetch_global_index!/2","doc":"","ref":"Orb.ToWasm.Context.html#fetch_global_index!/2"},{"type":"function","title":"Orb.ToWasm.Context.fetch_local_index!/2","doc":"","ref":"Orb.ToWasm.Context.html#fetch_local_index!/2"},{"type":"function","title":"Orb.ToWasm.Context.fetch_loop_identifier_index!/2","doc":"","ref":"Orb.ToWasm.Context.html#fetch_loop_identifier_index!/2"},{"type":"function","title":"Orb.ToWasm.Context.get_custom_type_index/2","doc":"","ref":"Orb.ToWasm.Context.html#get_custom_type_index/2"},{"type":"function","title":"Orb.ToWasm.Context.new/0","doc":"","ref":"Orb.ToWasm.Context.html#new/0"},{"type":"function","title":"Orb.ToWasm.Context.register_loop_identifier/2","doc":"","ref":"Orb.ToWasm.Context.html#register_loop_identifier/2"},{"type":"function","title":"Orb.ToWasm.Context.set_custom_type_index_lookup/2","doc":"","ref":"Orb.ToWasm.Context.html#set_custom_type_index_lookup/2"},{"type":"function","title":"Orb.ToWasm.Context.set_func_name_index_lookup/2","doc":"","ref":"Orb.ToWasm.Context.html#set_func_name_index_lookup/2"},{"type":"function","title":"Orb.ToWasm.Context.set_global_name_index_lookup/2","doc":"","ref":"Orb.ToWasm.Context.html#set_global_name_index_lookup/2"},{"type":"function","title":"Orb.ToWasm.Context.set_local_get_types/2","doc":"","ref":"Orb.ToWasm.Context.html#set_local_get_types/2"},{"type":"protocol","title":"Orb.ToWat","doc":"","ref":"Orb.ToWat.html"},{"type":"function","title":"Orb.ToWat.to_wat/2","doc":"Protocol for converting to WebAssembly text format (.wat)","ref":"Orb.ToWat.html#to_wat/2"},{"type":"type","title":"Orb.ToWat.t/0","doc":"All the types that implement this protocol.","ref":"Orb.ToWat.html#t:t/0"},{"type":"module","title":"Orb.ToWat.Helpers","doc":"","ref":"Orb.ToWat.Helpers.html"},{"type":"function","title":"Orb.ToWat.Helpers.do_type/1","doc":"","ref":"Orb.ToWat.Helpers.html#do_type/1"},{"type":"module","title":"Orb.TypeDefinition","doc":"","ref":"Orb.TypeDefinition.html"},{"type":"protocol","title":"Orb.TypeNarrowable","doc":"","ref":"Orb.TypeNarrowable.html"},{"type":"function","title":"Orb.TypeNarrowable.type_narrow_to/2","doc":"Protocol for narrowing types e.g. from Elixir.Integer to I32.","ref":"Orb.TypeNarrowable.html#type_narrow_to/2"},{"type":"type","title":"Orb.TypeNarrowable.t/0","doc":"All the types that implement this protocol.","ref":"Orb.TypeNarrowable.html#t:t/0"},{"type":"module","title":"Orb.U32.DSL","doc":"Unsigned 32-bit integer operators.\n\nSome operators like division have two variations in WebAssembly: signed and unsigned. By default math perform signed operations, but if you want unsigned math you can pass `U32` to `Orb.DSL.Defw.wasm_mode/1` like so: `wasm_mode(Orb.U32)`.","ref":"Orb.U32.DSL.html"},{"type":"function","title":"Orb.U32.DSL.//2","doc":"","ref":"Orb.U32.DSL.html#//2"},{"type":"function","title":"Orb.U32.DSL.</2","doc":"","ref":"Orb.U32.DSL.html#%3C/2"},{"type":"function","title":"Orb.U32.DSL.<=/2","doc":"","ref":"Orb.U32.DSL.html#%3C=/2"},{"type":"function","title":"Orb.U32.DSL.>/2","doc":"","ref":"Orb.U32.DSL.html#%3E/2"},{"type":"function","title":"Orb.U32.DSL.>=/2","doc":"","ref":"Orb.U32.DSL.html#%3E=/2"},{"type":"function","title":"Orb.U32.DSL.>>>/2","doc":"","ref":"Orb.U32.DSL.html#%3E%3E%3E/2"},{"type":"module","title":"Orb.VariableReference.Global","doc":"","ref":"Orb.VariableReference.Global.html"},{"type":"function","title":"Orb.VariableReference.Global.set/2","doc":"","ref":"Orb.VariableReference.Global.html#set/2"},{"type":"function","title":"Orb.VariableReference.Global.set_from_stack/1","doc":"","ref":"Orb.VariableReference.Global.html#set_from_stack/1"},{"type":"module","title":"Orb.VariableReference.Local","doc":"","ref":"Orb.VariableReference.Local.html"},{"type":"function","title":"Orb.VariableReference.Local.set/2","doc":"","ref":"Orb.VariableReference.Local.html#set/2"},{"type":"function","title":"Orb.VariableReference.Local.set_from_stack/1","doc":"","ref":"Orb.VariableReference.Local.html#set_from_stack/1"},{"type":"exception","title":"Orb.CustomType.InvalidError","doc":"","ref":"Orb.CustomType.InvalidError.html"},{"type":"exception","title":"Orb.Instruction.Call.ArgumentTypeError","doc":"","ref":"Orb.Instruction.Call.ArgumentTypeError.html"},{"type":"exception","title":"Orb.ModuleDefinition.FetchFuncError","doc":"","ref":"Orb.ModuleDefinition.FetchFuncError.html"},{"type":"exception","title":"Orb.ToWat.Error","doc":"","ref":"Orb.ToWat.Error.html"},{"type":"exception","title":"Orb.TypeCheckError","doc":"","ref":"Orb.TypeCheckError.html"},{"type":"extras","title":"Example: Introduction","doc":"# Introduction to Orb\n\n```elixir\nMix.install([\n  {:jason, \"~> 1.0\"},\n  {:orb, \"~> 0.0.48\"},\n  {:wasmex, \"~> 0.8.3\"}\n])\n```","ref":"01-intro.html"},{"type":"extras","title":"Define WebAssembly module using Orb - Example: Introduction","doc":"```elixir\ndefmodule CalculateMean do\n  use Orb\n\n  global do\n    @tally 0\n    @count 0\n  end\n\n  defw insert(n: I32) do\n    @tally = @tally + n\n    @count = @count + 1\n  end\n\n  defw calculate_mean(), I32 do\n    @tally / @count\n  end\nend\n```","ref":"01-intro.html#define-webassembly-module-using-orb"},{"type":"extras","title":"Run WebAssembly module using Wasmex - Example: Introduction","doc":"```elixir\n{:ok, pid} = Wasmex.start_link(%{bytes: Orb.to_wat(CalculateMean)})\nWasmex.call_function(pid, :insert, [3])\nWasmex.call_function(pid, :insert, [4])\nWasmex.call_function(pid, :insert, [5])\nWasmex.call_function(pid, :insert, [6])\nWasmex.call_function(pid, :insert, [7])\n{:ok, [5]} = Wasmex.call_function(pid, :calculate_mean, [])\n```","ref":"01-intro.html#run-webassembly-module-using-wasmex"},{"type":"extras","title":"Example: Temperature Converter","doc":"# Orb Temperature Converter\n\n```elixir\nMix.install([\n  {:orb, \"~> 0.0.48\"},\n  {:wasmex, \"~> 0.8.3\"}\n])\n```","ref":"temperature-converter.html"},{"type":"extras","title":"Define our temperature converter module using Orb - Example: Temperature Converter","doc":"```elixir\ndefmodule TemperatureConverter do\n  use Orb\n\n  Orb.I32.export_enum([:celsius, :fahrenheit])\n\n  global do\n    @mode 0\n  end\n\n  defw celsius_to_fahrenheit(celsius: F32), F32 do\n    celsius * (9.0 / 5.0) + 32.0\n  end\n\n  defw fahrenheit_to_celsius(fahrenheit: F32), F32 do\n    (fahrenheit - 32.0) * (5.0 / 9.0)\n  end\n\n  defw convert_temperature(temperature: F32), F32 do\n    if @mode === @celsius do\n      celsius_to_fahrenheit(temperature)\n    else\n      fahrenheit_to_celsius(temperature)\n    end\n  end\n\n  defw set_mode(mode: I32) do\n    @mode = mode\n  end\nend\n```","ref":"temperature-converter.html#define-our-temperature-converter-module-using-orb"},{"type":"extras","title":"We can compile this module to the WebAssembly text format - Example: Temperature Converter","doc":"```elixir\nOrb.to_wat(TemperatureConverter)\n|> IO.puts()\n```","ref":"temperature-converter.html#we-can-compile-this-module-to-the-webassembly-text-format"},{"type":"extras","title":"We can run the WebAssembly module in Elixir using Wasmex - Example: Temperature Converter","doc":"```elixir\n{:ok, pid} = Wasmex.start_link(%{bytes: Orb.to_wat(TemperatureConverter)})\n\n# Wasmex.call_function(pid, :fahrenheit_to_celsius, [72.0])\n\nWasmex.call_function(pid, :convert_temperature, [72.0])\n|> IO.inspect()\n\nWasmex.call_function(pid, :set_mode, [1])\n\nWasmex.call_function(pid, :convert_temperature, [72.0])\n|> IO.inspect()\n```","ref":"temperature-converter.html#we-can-run-the-webassembly-module-in-elixir-using-wasmex"},{"type":"extras","title":"Example: Mime Type","doc":"# Orb Mime Type\n\n```elixir\nMix.install([\n  :orb,\n  {:wasmex, \"~> 0.8.3\"},\n  {:mime, \"~> 2.0.5\"},\n  {:kino, \"~> 0.12.0\"}\n])\n```","ref":"mime-type.html"},{"type":"extras","title":"Define our Mime Type module using Orb - Example: Mime Type","doc":"```elixir\ndefmodule MimeType do\n  use Orb\n\n  defw(txt, I32.UnsafePointer, do: MIME.type(\"txt\"))\n  defw(json, I32.UnsafePointer, do: MIME.type(\"json\"))\n  defw(html, I32.UnsafePointer, do: MIME.type(\"html\"))\n  defw(css, I32.UnsafePointer, do: MIME.type(\"css\"))\n  defw(wasm, I32.UnsafePointer, do: MIME.type(\"wasm\"))\n  defw(epub, I32.UnsafePointer, do: MIME.type(\"epub\"))\n  defw(rss, I32.UnsafePointer, do: MIME.type(\"rss\"))\n  defw(atom, I32.UnsafePointer, do: MIME.type(\"atom\"))\n  defw(csv, I32.UnsafePointer, do: MIME.type(\"csv\"))\n  defw(woff2, I32.UnsafePointer, do: MIME.type(\"woff2\"))\n  defw(pdf, I32.UnsafePointer, do: MIME.type(\"pdf\"))\n  defw(js, I32.UnsafePointer, do: \"application/javascript\")\n  defw(xml, I32.UnsafePointer, do: \"application/xml\")\n  defw(sqlite, I32.UnsafePointer, do: \"application/vnd.sqlite3\")\nend\n```","ref":"mime-type.html#define-our-mime-type-module-using-orb"},{"type":"extras","title":"We can run the WebAssembly module in Elixir using Wasmex - Example: Mime Type","doc":"```elixir\nfile_extension = \"html\"\n\n{:ok, pid} = Wasmex.start_link(%{bytes: Orb.to_wat(MimeType)})\n{:ok, memory} = Wasmex.memory(pid)\n{:ok, store} = Wasmex.store(pid)\nbyte_count = Wasmex.Memory.size(store, memory)\nbytes = Wasmex.Memory.read_binary(store, memory, 0, byte_count)\n\ncase Wasmex.call_function(pid, file_extension, []) do\n  {:ok, [str_ptr]} ->\n    slice = binary_slice(bytes, str_ptr..-1//1)\n\n    for(< >, do: chunk)\n    |> Enum.take_while(fn c -> c != 0 end)\n    |> List.to_string()\n\n  {:error, _} ->\n    nil\nend\n```","ref":"mime-type.html#we-can-run-the-webassembly-module-in-elixir-using-wasmex"},{"type":"extras","title":"Example: YouTube URL Parser","doc":"# Orb YouTube URL parser\n\n```elixir\nMix.install([\n  {:orb, \"~> 0.0.48\"},\n  {:wasmex, \"~> 0.8.3\"},\n  {:mime, \"~> 2.0.5\"},\n  {:kino, \"~> 0.12.0\"}\n])\n```","ref":"youtube-url-parser.html"},{"type":"extras","title":"Define a helper module for parsing characters - Example: YouTube URL Parser","doc":"```elixir\ndefmodule CharParser do\n  use Orb\n\n  def may(chars, offset_mut_ref) when is_list(chars) do\n    Orb.snippet do\n      chars\n      |> Enum.with_index(fn char, index ->\n        Memory.load!(I32.U8, offset_mut_ref.read + index) === char\n      end)\n      |> Enum.reduce(&I32.band/2)\n      |> if do\n        offset_mut_ref.read + length(chars)\n        offset_mut_ref.write\n      end\n    end\n  end\n\n  def must(chars, offset_mut_ref, fail_instruction) when is_list(chars) do\n    Orb.snippet do\n      unless may(chars, offset_mut_ref) do\n        fail_instruction\n      end\n    end\n  end\n\n  # Future API?\n  # CharParser.seq(mut!(input_offset), return(0), must: ~c|http|, may: ~c|s|, must: ~c|:|, may: ~c|//|)\n\n  # Future API?\n  # CharParser.one_of([\n  #   [may: ~c|www.|, must: ~c|youtube.com|], must: ~c|/watch?v=|],\n  #   [must: ~c|youtu.be/|]\n  # ], mut!(input_offset), return(0))\n\n  def print(ast) do\n    IO.puts(Orb.ToWat.to_wat(ast, \"\") |> IO.iodata_to_binary())\n  end\nend\n```","ref":"youtube-url-parser.html#define-a-helper-module-for-parsing-characters"},{"type":"extras","title":"Define our module using Orb - Example: YouTube URL Parser","doc":"```elixir\ndefmodule YouTubeURLParser do\n  use Orb\n\n  Memory.pages(2)\n  defp start_page_offset(), do: 1\n\n  defw(input_offset(), I32, do: start_page_offset() * Memory.page_byte_size())\n\n  defw parse(), I32, input_offset: I32.UnsafePointer do\n    input_offset = start_page_offset() * Memory.page_byte_size()\n\n    # input_offset[~c|http|]\n\n    Control.block OptionalScheme do\n      CharParser.must(~c|http|, mut!(input_offset), OptionalScheme.break())\n      # unless CharParser.may(~c|http|, mut!(input_offset)), do: OptionalScheme.break()\n      CharParser.may(~c|s|, mut!(input_offset))\n      CharParser.must(~c|:|, mut!(input_offset), OptionalScheme.break())\n    end\n\n    CharParser.may(~c|//|, mut!(input_offset))\n\n    Control.block Final do\n      Control.block ShortDomain do\n        input_offset =\n          Orb.Stack.push input_offset do\n            Control.block FullDomain do\n              CharParser.may(~c|www.|, mut!(input_offset))\n              CharParser.must(~c|youtube.com|, mut!(input_offset), FullDomain.break())\n              CharParser.must(~c|/watch?v=|, mut!(input_offset), FullDomain.break())\n              unless input_offset[at!: 0], do: FullDomain.break()\n              Final.break()\n            end\n          end\n\n        CharParser.must(~c|youtu.be/|, mut!(input_offset), ShortDomain.break())\n        unless input_offset[at!: 0], do: ShortDomain.break()\n        Final.break()\n      end\n\n      return(0)\n    end\n\n    input_offset\n  end\n\n  def run(input) do\n    {:ok, pid} = Wasmex.start_link(%{bytes: Orb.to_wat(__MODULE__)})\n    {:ok, memory} = Wasmex.memory(pid)\n    {:ok, store} = Wasmex.store(pid)\n\n    {:ok, [input_offset]} = Wasmex.call_function(pid, :input_offset, [])\n    Wasmex.Memory.write_binary(store, memory, input_offset, input <> \"\\0\")\n\n    {:ok, [result]} = Wasmex.call_function(pid, :parse, [])\n\n    case result do\n      0 ->\n        nil\n\n      offset ->\n        Wasmex.Memory.read_string(store, memory, offset, 255)\n        |> String.replace_trailing(\"\\0\", \"\")\n    end\n  end\nend\n```","ref":"youtube-url-parser.html#define-our-module-using-orb"},{"type":"extras","title":"We can run the WebAssembly module in Elixir using Wasmex - Example: YouTube URL Parser","doc":"```elixir\n# source = Orb.to_wat(YouTubeURLParser)\n# File.write!(Path.join([File.cwd!(), \"Desktop\", \"youtube.wat\"]), source)\n\n[\n  \"http://www.youtube.com/watch?v=hello\",\n  \"https://www.youtube.com/watch?v=hello\",\n  \"https://youtube.com/watch?v=hello\",\n  \"https://youtu.be/hello\",\n  \"//www.youtube.com/watch?v=hello\",\n  \"//youtu.be/hello\",\n  \"www.youtube.com/watch?v=hello\",\n  # These should fail\n  \"www.youtu.be/watch?v=hello\",\n  \"https://www.youtube.com/watch\",\n  \"https://www.youtube.com/watch?v=\",\n  \"https://youtu.be/\",\n  \"hello\"\n]\n|> Enum.map(&YouTubeURLParser.run/1)\n```","ref":"youtube-url-parser.html#we-can-run-the-webassembly-module-in-elixir-using-wasmex"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}